//   Copyright 2020 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package com.example.app;

import com.esri.arcgisruntime.geometry.Point;
import com.esri.arcgisruntime.geometry.PointCollection;
import com.esri.arcgisruntime.geometry.Polygon;
import com.esri.arcgisruntime.geometry.Polyline;
import com.esri.arcgisruntime.geometry.SpatialReferences;
import com.esri.arcgisruntime.mapping.view.Graphic;
import com.esri.arcgisruntime.mapping.view.GraphicsOverlay;
import com.esri.arcgisruntime.symbology.SimpleFillSymbol;
import com.esri.arcgisruntime.symbology.SimpleLineSymbol;
import com.esri.arcgisruntime.symbology.SimpleMarkerSymbol;

import com.esri.arcgisruntime.ArcGISRuntimeEnvironment;
import com.esri.arcgisruntime.mapping.ArcGISMap;
import com.esri.arcgisruntime.mapping.BasemapStyle;
import com.esri.arcgisruntime.mapping.Viewpoint;
import com.esri.arcgisruntime.mapping.view.MapView;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

public class App extends Application {

  private MapView mapView;

  public static void main(String[] args) {
    Application.launch(args);
  }

  @Override
  public void start(Stage stage) {

    // set the title and size of the stage and show it
    stage.setTitle("Add a point line and polygon tutorial");
    stage.setWidth(800);
    stage.setHeight(700);
    stage.show();

    // create a JavaFX scene with a stack pane as the root node
    // and add it to the scene
    StackPane stackPane = new StackPane();
    Scene scene = new Scene(stackPane);
    stage.setScene(scene);

    // Note: it is not best practice to store API keys in source code.
    // The API key is referenced here for the convenience of this tutorial.
    String yourApiKey = "YOUR_API_KEY";
    ArcGISRuntimeEnvironment.setApiKey(yourApiKey);

    // create a map view to display the map and add it to the stack pane
    mapView = new MapView();
    stackPane.getChildren().add(mapView);
    ArcGISMap map = new ArcGISMap(BasemapStyle.ARCGIS_TOPOGRAPHIC);

    // set the map on the map view
    mapView.setMap(map);
    mapView.setViewpoint(new Viewpoint(34.02700, -118.80543, 72223.819286));

    // create a graphics overlay and add it to the map view
    GraphicsOverlay graphicsOverlay = new GraphicsOverlay();
    mapView.getGraphicsOverlays().add(graphicsOverlay);

    // create a point geometry with a location and spatial reference
    Point point = new Point(-118.80657463861, 34.0005930608889, SpatialReferences.getWgs84());
    // create an opaque orange point symbol with a opaque blue outline symbol
    SimpleMarkerSymbol simpleMarkerSymbol =
      new SimpleMarkerSymbol(SimpleMarkerSymbol.Style.CIRCLE, Color.ORANGE, 10);
    SimpleLineSymbol blueOutlineSymbol =
      new SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.BLUE, 2);

    simpleMarkerSymbol.setOutline(blueOutlineSymbol);

    // create a graphic with the point geometry and symbol
    Graphic pointGraphic = new Graphic(point, simpleMarkerSymbol);

    // add the point graphic to the graphics overlay
    graphicsOverlay.getGraphics().add(pointGraphic);

    // create a point collection with a spatial reference, and add three points to it
    PointCollection polylinePoints = new PointCollection(SpatialReferences.getWgs84());
    polylinePoints.add(new Point(-118.821527826096, 34.0139576938577));
    polylinePoints.add(new Point(-118.814893761649, 34.0080602407843));
    polylinePoints.add(new Point(-118.808878330345, 34.0016642996246));
    // create a polyline geometry from the point collection
    Polyline polyline = new Polyline(polylinePoints);

    // create an opaque blue line symbol for the polyline
    SimpleLineSymbol polylineSymbol =
      new SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.BLUE, 3);

    // create a polyline graphic with the polyline geometry and symbol
    Graphic polylineGraphic = new Graphic(polyline, polylineSymbol);

    // add the polyline graphic to the graphics overlay
    graphicsOverlay.getGraphics().add(polylineGraphic);

    // create a point collection with a spatial reference, and add five points to it
    PointCollection polygonPoints = new PointCollection(SpatialReferences.getWgs84());
    polygonPoints.add(new Point(-118.818984489994, 34.0137559967283));
    polygonPoints.add(new Point(-118.806796597377, 34.0215816298725));
    polygonPoints.add(new Point(-118.791432890735, 34.0163883241613));
    polygonPoints.add(new Point(-118.795966865355, 34.0085648646355));
    polygonPoints.add(new Point(-118.808558110679, 34.0035027131376));
    // create a polygon geometry from the point collection
    Polygon polygon = new Polygon(polygonPoints);

    // create an orange-red fill symbol with 20% transparency and the opaque blue simple line symbol
    SimpleFillSymbol polygonFillSymbol =
      new SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, Color.web("#FF4500", .8), blueOutlineSymbol);

    // create a polygon graphic from the polygon geometry and symbol
    Graphic polygonGraphic = new Graphic(polygon, polygonFillSymbol);
    // add the polygon graphic to the graphics overlay
    graphicsOverlay.getGraphics().add(polygonGraphic);

  }

  /**
   * Stops and releases all resources used in application.
   */
  @Override
  public void stop() {
    if (mapView != null) {
      mapView.dispose();
    }
  }

}

